package com.sundata.internalevaluation.calc.calcUnit;

import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONException;
import cn.hutool.json.JSONUtil;
import com.sundata.common.util.JsonUtil;
import com.sundata.internalevaluation.calc.calcUnit.interfaces.InterfaceRunning;
import com.sundata.internalevaluation.calc.exception.CalcConfigException;
import com.sundata.internalevaluation.calc.exception.SendRequestException;
import com.sundata.internalevaluation.calc.model.CalcResult;
import com.sundata.internalevaluation.calc.model.CalcUnit;
import com.sundata.internalevaluation.calc.model.finals.CalcType;
import com.sundata.internalevaluation.calc.util.InvokeUtil;
import com.sundata.internalevaluation.calc.util.JdbcUtils;
import com.sundata.internalevaluation.configuration.model.Interface;
import com.sundata.internalevaluation.configuration.model.InterfaceParam;
import com.sundata.internalevaluation.script.TemplateUtil;
import org.apache.commons.text.StringSubstitutor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.JdbcTemplate;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Created by IntelliJ IDEA.
 *
 * @author JoeLazy
 * @date 2025-02-12 10:40:49
 * @description: 接口计算单元
 */
public class InterfaceCalcUnit extends CalcUnit {

    private static final Logger log = LoggerFactory.getLogger(InterfaceCalcUnit.class);


    private final Interface interfaceModel;


    /**
     * 创建数据单元的绝对对象，对象必须包含如下参数
     *
     * @param calcCode    计算对象编号
     * @param calcName    计算对象名称
     * @param calcType    计算类型
     * @param initContext 计算单元初始化参数
     */
    public InterfaceCalcUnit(String calcCode, String calcName, CalcType calcType, Map<String, Object> initContext, Interface interfaceModel) {
        super(calcCode, calcName, calcType, initContext);
        this.interfaceModel = interfaceModel;
    }

    /**
     * 判断是否已经计算过数据了
     *
     * @param calculateInstanceNumber 计算流水号
     * @return 是否计算过 true 计算过 false 没有计算过
     */
    @Override
    public boolean isCalcFinished(String calculateInstanceNumber) {
        return false;
    }

    /**
     * 初始化计算结果的方法，如果已经计算过，在实现过程中，应当在此方法中根据计算流水号重新初始化 resultContext 结果对象，为其他依赖对象做准备
     * 若明明计算过本单元但再次计算时没有初始化该对象，则计算依赖出现问题无法定位与处理
     *
     * @param calculateInstanceNumber 计算流水号
     */
    @Override
    public void initResultContext(String calculateInstanceNumber) {

    }

    /**
     * 根据节点配置获取源节点；
     *
     * @return 所有源头节点
     */
    @Override
    public List<CalcUnit> getSourceCalcUnits() {
        return List.of();
    }

    /**
     * 计算之后的方法，可实现为空
     *
     * @param context
     */
    @Override
    public void afterCalc(Map<String, Object> context) {

    }

    /**
     * 计算之前，可实现空
     *
     * @param context
     */
    @Override
    public void beforeCalc(Map<String, Object> context) {

    }

    /**
     * 必须实现的主体计算内容
     *
     * @param thisResult              本计算单元的结果
     * @param calculateInstanceNumber 计算流水号
     * @param context                 节点计算参数清单
     * @param sourceResults           源头计算节点的结果
     */
    @Override
    public void calc(CalcResult<String, Object> thisResult,
                     String calculateInstanceNumber,
                     Map<String, Object> context,
                     Map<CalcUnit, CalcResult<String, Object>> sourceResults) {

        log.debug("计算{}：编号:{}，名称:{}，流水号:{}", getCalcType().getName(), getCalcCode(), getCalcName(), calculateInstanceNumber);
        // 获取新流水号，每次请求接口必须不一样 TODO 根据实际情况生成全局流水号
//        String globalReqNo = AdminUtil.getNextId("cretSeqno");
        String globalReqNo = UUID.randomUUID().toString();
        context.put("globalReqNo",globalReqNo);
        // creditSeqNo 查询征信的流水号 如果不存在就重新赋值 只有征信的请求才会用到
        context.putIfAbsent("creditSeqNo",globalReqNo);

        // 接口内容类型
        String reqContentTypeCode = this.interfaceModel.getReqContentTypeCode();
        // 接口请求类型
        String interfaceTypeCode = this.interfaceModel.getInterfaceTypeCode();

        // 请求类型不为 http或者 https   或者   请求体不为 json
        if (!interfaceTypeCode.startsWith("http") || !"json".equals(reqContentTypeCode)) {
            // TODO Socket 请求
            throw new CalcConfigException("目前仅支持http和https的请求和json请求体");
        }

        // 请求参数实体列表
        List<InterfaceParam> paramList = interfaceModel.getParamList();
        Map<String, String> headersMap = new HashMap<>();
        Map<String, Object> paramMap = new HashMap<>();

        // 封装请求参数
        paramList.forEach(
                param -> {
                    Object result = getParamValue(param, context);
                    String isHeader = param.getIsHeader();
                    if ("true".equals(isHeader)) {
                        // 请求头信息
                        headersMap.put(param.getParamObjName(), JsonUtil.toJSONString(result));
                    } else {
                        // 请求参数
                        paramMap.put(param.getParamObjName(), result);
                    }
                }
        );


        // http请求或者 https请求
        if (interfaceTypeCode.startsWith("http")) {
            // 拼接url  其中可能存在变量
            String url = interfaceTypeCode + "://" +
                    interfaceModel.getInterfaceIp() + ":" +
                    interfaceModel.getInterfacePort() +"/"+
                    interfaceModel.getInterfaceUrl();

            Map<String,Object> allMap = new HashMap<>(headersMap);
            allMap.putAll(paramMap);
            StringSubstitutor sub  = new StringSubstitutor(allMap);
            // 替换url中的变量
            url = sub.replace(url);

            // 请求体模板和完整请求体
            String reqBodyTemplate = interfaceModel.getReqBodyTemplate();
            String jsonString = "";



            // 替换请求报文模板中的变量
            if (reqBodyTemplate != null && !reqBodyTemplate.trim().isEmpty()) {
                jsonString = TemplateUtil.execute("interface-temp-bodyTemp" + getCalcCode(),reqBodyTemplate,allMap);
            }

            // 声明 接口参数处理的对象
            InterfaceRunning interfaceRunning;

            // 加密逻辑
            String encodeLogic = interfaceModel.getReqEncryptionLogic();
            if (StrUtil.isNotBlank(encodeLogic)) {
                interfaceRunning = SpringUtil.getBean(encodeLogic);
                if (interfaceRunning != null) {
                    jsonString = interfaceRunning.encode(jsonString);
                }
            }

            //  声明 HttpRequest 对象
            HttpRequest httpRequest;

            // 拿到响应体
            String responseStr;
            String interfaceNo = interfaceModel.getInterfaceNo();
            String reqMethod = interfaceModel.getReqMethod();
            if ("get".equals(reqMethod)) {
                httpRequest = HttpUtil.createGet(url);
            } else {
                httpRequest = HttpUtil.createPost(url);
            }

            // 设置请求头和请求体
            httpRequest.addHeaders(headersMap);
            if (jsonString != null && !jsonString.trim().isEmpty()) {
                httpRequest.body(jsonString);
            }
            log.debug("接口-[{}({})]-请求地址-[{}]-请求body报文：{}",getCalcName(),getCalcCode(),url,jsonString);
            try (HttpResponse httpResponse = httpRequest.execute()) {
                responseStr = httpResponse.body();
                log.debug("接口-[{}({})]-请求地址-[{}]-响应body报文：{}",getCalcName(),getCalcCode(),url,responseStr);
            }catch (Exception e){
                String errMsg = StrUtil.format("接口-[{}({})]-请求地址-[{}] - 请求出现异常：{}", getCalcName(), getCalcCode(), url, e.getMessage());
                log.error(errMsg,e);
                throw new SendRequestException("请求发送失败：["+url+"]", e);
            }

            // 判断返回值类型
            String respParamTypeCode = interfaceModel.getRespParamTypeCode();
            if ("json".equals(respParamTypeCode)){
                try {
                    responseStr = JSONUtil.toJsonPrettyStr(responseStr);
                } catch (JSONException e) {
                    log.error("接口-[{}({})]-请求地址-[{}] - 返回值不是程序能解析的json字符串，请检查！", getCalcName(), getCalcCode(), url);
                    throw new CalcConfigException("返回值不是程序能解析的json字符串，请检查！",e);
                }
            }
            // TODO 可继续增加其他类型
            else
            {
                throw new CalcConfigException("接口-目前只支持 json 返回报文");
            }



            // 解密逻辑
            String decryptionLogic = interfaceModel.getDecryptionLogic();
            if (null != decryptionLogic && !decryptionLogic.trim().isEmpty()) {
                interfaceRunning = SpringUtil.getBean(decryptionLogic);
                if (interfaceRunning != null) {
                    responseStr = interfaceRunning.decode(responseStr);
                    log.debug("接口-[{}({})]-请求地址-[{}]-解密后数据：{}",getCalcName(),getCalcCode(),url,responseStr);
                }
            }

            // 解密后处理逻辑
            String processLogic = interfaceModel.getProcessLogic();
            if (processLogic != null && !processLogic.trim().isEmpty()) {
                interfaceRunning = SpringUtil.getBean(processLogic);
                if (interfaceRunning != null) {
                    responseStr = interfaceRunning.afterDecode(interfaceNo, responseStr, calculateInstanceNumber);
                    log.debug("接口-[{}({})]-请求地址-[{}]-解析后数据：{}",getCalcName(),getCalcCode(),url,responseStr);
                }
            }

            /*
            responseStr 一定是如下结果(键为字符串，值为数组的Json字符串)，否则后期计算会报错
            {
                "propA":[
                    {
                        "propA1":"valueA1-1",
                        "propA2":"valueA1-2",
                        "propA3":"valueA1-3",
                        ...
                    },
                    {
                        "propA1":"valueA2-1",
                        "propA2":"valueA2-2",
                        "propA3":"valueA2-3",
                    },
                    ...
                ],
                "propB":[
                    {
                        "propB1":"valueB1-1",
                        "propB2":"valueB1-2",
                        "propB3":"valueB1-3",
                        ...
                    },
                    {
                        "propB1":"valueB2-1",
                        "propB2":"valueB2-2",
                        "propB3":"valueB2-3",
                    },
                    ...
                ],
                ...
            }
             */
            // 保存结果
            thisResult.put(getCalcCode(), responseStr);
        }
    }


    /**
     * 获取请求参数的值
     * @param interfaceParam 请求参数实体
     * @param context 调用计算单元传入的值
     * @return 参数结果
     */
    private Object getParamValue(InterfaceParam interfaceParam, Map<String, Object> context) {
        // 参数获取类型
        String paramRetrievalTypeCode = interfaceParam.getParamRetrievalTypeCode();

        // 参数获取内容
        String paramRetrievalTypeCont = interfaceParam.getParamRetrievalTypeCont();

        Object result = null;

        JdbcTemplate jdbcTemplate = JdbcUtils.getJdbcTemplate();

        // 获取参数值
        switch (paramRetrievalTypeCode) {
            // 固定值
            case "fixed":
                result = paramRetrievalTypeCont;
                break;
            // java
            //   获取参数内容
            case "java":
                try {
                    result = InvokeUtil.invokeMethod(paramRetrievalTypeCont);
                } catch (Exception e) {
                    String errorMsg = StrUtil.format("接口(参数)-[{}({})]-通过Java获取参数失败，对应方法为：[{}]", getCalcName(), interfaceParam.getParamObjName(), paramRetrievalTypeCont);
                    log.error(errorMsg);
                    throw new CalcConfigException(errorMsg,e);
                }
                break;
            // SQL-Map
            case "sqlmap":
                String sqlMap = TemplateUtil.execute("interfaceParam-temp-sqlmap-" + getCalcCode(), paramRetrievalTypeCont, context);
                log.debug("通过SQL-Map获取接口参数：{}",sqlMap);
                result = jdbcTemplate.queryForMap(sqlMap);
                break;
            // SQL-List
            case "sqllist":
                String sqlList = TemplateUtil.execute("interfaceParam-temp-sqllist-" + getCalcCode(), paramRetrievalTypeCont, context);
                log.debug("通过SQL-Map获取接口参数：{}",sqlList);
                result = jdbcTemplate.queryForList(sqlList);
                break;
            // SQL-String
            case "sqlstring":
                String sqlString = TemplateUtil.execute("interfaceParam-temp-sqlstring-" + getCalcCode(), paramRetrievalTypeCont, context);
                log.debug("通过SQL-Map获取接口参数：{}",sqlString);
                result = jdbcTemplate.queryForObject(sqlString, Object.class);
                break;
            case "callerInput":
                result = TemplateUtil.execute("interfaceParam-temp-callerInput-"+getCalcCode(),paramRetrievalTypeCont, context);
                break;
        }


        String paramTypeCode = interfaceParam.getParamTypeCode();
        if ("string".equals(paramTypeCode)) {
            if (result instanceof String) {

            } else {
                result = JsonUtil.toJSONString(result);
            }
        } else if ("number".equals(paramTypeCode)) {
            if (result instanceof Number) {
                if (result instanceof Integer) {
                    result = ((Number) result).intValue();
                } else if (result instanceof Long) {
                    result = ((Number) result).longValue();
                } else if (result instanceof Float) {
                    result = ((Number) result).floatValue();
                } else if (result instanceof Double) {
                    result = ((Number) result).doubleValue();
                } else if (result instanceof Short) {
                    result = ((Number) result).shortValue();
                } else if (result instanceof Byte) {
                    result = ((Number) result).byteValue();
                } else {
                    throw new CalcConfigException("数据类型不匹配：计算机过不为'数字'");
                }
            } else {
                throw new CalcConfigException("数据类型不匹配：计算机过不为'数字'");
            }
        } else {
            throw new CalcConfigException("返回值类型有误，目前仅有 数字和字符串");
        }
        return result;
    }
}
