package com.sundata.internalevaluation.calc.running;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.sundata.internalevaluation.calc.exception.CalcConfigException;
import com.sundata.internalevaluation.calc.exception.RequestParamException;
import com.sundata.internalevaluation.calc.exception.ResponseDataException;
import com.sundata.internalevaluation.calc.exception.SendRequestException;
import com.sundata.internalevaluation.calc.model.CalcResult;
import com.sundata.internalevaluation.calc.model.CalcUnit;
import com.sundata.internalevaluation.calc.running.threads.CalcRecursiveTask;
import com.sundata.internalevaluation.calc.running.threads.CalcTaskResult;
import com.sundata.internalevaluation.calc.running.threads.CalcUnitThreadFactory;
import com.sundata.internalevaluation.calc.running.threads.fianl.CalcTaskResultType;
import groovy.lang.MissingPropertyException;
import org.jgrapht.Graph;
import org.jgrapht.graph.DefaultDirectedGraph;
import org.jgrapht.graph.DefaultEdge;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentSkipListSet;
import java.util.concurrent.ForkJoinPool;

public class CalcRunning {
    private static final Logger log = LoggerFactory.getLogger(CalcRunning.class);


    /**
     * 正在运行的计算流水号
     */
    private static final ConcurrentSkipListSet<String> runningList = new ConcurrentSkipListSet<>();

    public static ConcurrentSkipListSet<String> getRunningList(){
        return runningList;
    }


    /**
     * 计算方法
     * @param calculateInstanceNumber 计算流水号
     * @param calcUnit 集体的计算单元
     * @param context 初始化参数
     * @return 计算结果
     */
    public CalcTaskResult startCalc(String calculateInstanceNumber, CalcUnit calcUnit, Map<String, Object> context) {


        Iterator<Map.Entry<String, Object>> iterator = context.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, Object> entry = iterator.next();
            Object value = entry.getValue();
            if (value == null || "null".equals(value)){
                iterator.remove();
            }
        }

        // 流水号为空  生成流水号
        if (StrUtil.isEmpty(calculateInstanceNumber)) {
            calculateInstanceNumber = IdUtil.getSnowflakeNextIdStr();
        }


        // 计算结果
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("calculateInstanceNumber",calculateInstanceNumber);

        // 返回值对象
        CalcTaskResult result = new CalcTaskResult(CalcTaskResultType.SUCCESS, "200", "计算成功！", resultMap);

        if (runningList.contains(calculateInstanceNumber)) {
            log.warn("正在进行计算，请不要重复计算");
            result.setResultType(CalcTaskResultType.Ignore);
            result.setResult("正在进行计算，请不要重复计算");
            return result;
        }
        runningList.add(calculateInstanceNumber);


        // 创建自定义线程工厂
        CalcUnitThreadFactory factory = new CalcUnitThreadFactory(calculateInstanceNumber);

        // 创建 ForkJoinPool 并使用自定义线程工厂
        ForkJoinPool customPool = new ForkJoinPool(20, factory, null, true);

        // ？？
        final ConcurrentHashMap<String, Object> content = new ConcurrentHashMap<>(context);
        // 当前节点计算结果
        final ConcurrentHashMap<CalcUnit, CalcResult<String, Object>> results = new ConcurrentHashMap<>();
        // 子级节点计算结果
        final ConcurrentHashMap<CalcUnit, CalcResult<String, Object>> childResults = new ConcurrentHashMap<>();

        // 图结构对象
        final Graph<CalcUnit, DefaultEdge> graph = new DefaultDirectedGraph<>(DefaultEdge.class);

        graph.addVertex(calcUnit);
        graphAddVertex(graph,calcUnit,calcUnit.getSourceCalcUnits());

        // 提交任务到自定义线程池
        CalcResult<String ,Object> resultMain = null;
        try {
            CalcRecursiveTask task = new CalcRecursiveTask(calculateInstanceNumber, calcUnit, content, graph, results,childResults);
            resultMain = customPool.submit(task).join();
            resultMap.putAll(resultMain);
        } catch (Exception e) {
            result.setCode("500");
            String message = e.getMessage();
            log.error(message,e);
            if (e instanceof CalcConfigException){
                result.setResultType(CalcTaskResultType.ConfigProblem);
            }else if (e instanceof SendRequestException){
                result.setResultType(CalcTaskResultType.RequestProblem);
            }else if (e instanceof ResponseDataException){
                result.setResultType(CalcTaskResultType.ResponseProblem);
            }else if (e instanceof RequestParamException || e instanceof MissingPropertyException){
                result.setResultType(CalcTaskResultType.ParameterProblem);
            }else{
                result.setResultType(CalcTaskResultType.ProceduralIssues);
                message = "程序内部异常";
            }
            StringWriter sw = new StringWriter();
            e.printStackTrace(new PrintWriter(sw, true));
            resultMap.put("errorDetailMessage",sw.toString());
            result.setMessage(message);
            return result;
        } finally {
            System.out.println("关闭计算");
            customPool.shutdown();
        }
        // 删除正在运行的处理过程
        runningList.remove(calculateInstanceNumber);
        return result;
    }

    /**
     * 递归获取并构建图形结构
     * @param graph 图对象
     * @param calcUnit 被计算的单元
     * @param calcUnits 源头计算单元
     */
    private void graphAddVertex(Graph<CalcUnit, DefaultEdge> graph, CalcUnit calcUnit, List<CalcUnit> calcUnits) {
        if (calcUnits != null && !calcUnits.isEmpty()) {
            calcUnits.forEach(c -> {
                graph.addVertex(c);
                graph.addEdge(c, calcUnit);
                graphAddVertex(graph,c,c.getSourceCalcUnits());
            });
        }
    }

}
