package com.sundata.internalevaluation.configuration.service;

import com.sundata.common.exception.BusinessException;
import com.sundata.common.util.StringUtil;
import com.sundata.internalevaluation.configuration.mapper.RuleMapper;
import com.sundata.internalevaluation.configuration.mapper.RuleSetMapper;
import com.sundata.internalevaluation.configuration.model.Rule;
import com.sundata.internalevaluation.configuration.model.RuleIndex;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

@Service
public class RuleService {

    private static final Logger log = LoggerFactory.getLogger(RuleService.class);
    @Autowired
    private RuleMapper ruleMapper;

    @Autowired
    private RuleSetMapper ruleSetMapper;


    /**
     * 查询所有记录
     *
     * @return 返回集合，没有返回空List
     */
    public List<Rule> listAll(Rule rule) {
    	return ruleMapper.listAll(rule);
    }


    /**
     * 根据主键查询
     *
     * @param ruleNo 主键
     * @return 返回记录，没有返回null
     */
    @Transactional
    public Rule getById(String ruleNo) {
//        List<String> indexNoList = sysReqRuleMapper.selectIndexByRuleNo(ruleNo);
        Rule resEntity = ruleMapper.getById(ruleNo);
//        resEntity.setIndexNoList(indexNoList);
        return resEntity;
    }
	

    /**
     * 新增，忽略null字段
     *
     * @param rule 新增的记录
     * @return 返回影响行数
     */
    public int insertIgnoreNull(Rule rule) {
        String ruleNo = rule.getRuleNo();
        if (StringUtil.isEmpty(ruleNo)) {
            throw new BusinessException("规则编码不能为空.");
        }
        try {
            ruleMapper.deleteIndexByRuleNo(ruleNo);
            List<RuleIndex> ruleIndexList = rule.getRuleIndexList();
            ruleIndexList.forEach(ruleIndex -> ruleIndex.setRuleNo(ruleNo));
//            List<RuleIndex> list = rule.getIndexNoList().stream().map(indexNo -> new RuleIndex(rule.getRuleNo(), indexNo)).collect(Collectors.toList());
            insertIndexList(ruleIndexList);
            return ruleMapper.insertIgnoreNull(rule);
        } catch (DuplicateKeyException e) {
            throw new BusinessException("规则编码已存在，请重新输入.");
        }
    }


    /**
     * 插入规则指标关联信息
     * @param list 关联列表
     * @return 影响行数
     */
    public int insertIndexList(List<RuleIndex> list){
        if (list == null || list.isEmpty()) {
            return 0;
        }else {
            AtomicInteger rows= new AtomicInteger(0);
            list.forEach( ruleIndex -> {
                rows.getAndAdd(ruleMapper.insertIndex(ruleIndex));
            });
            return rows.get();
        }
    }
	


    /**
     * 修改，忽略null字段
     *
     * @param rule 修改的记录
     * @return 返回影响行数
     */
    @Transactional
    public int updateIgnoreNull(Rule rule) {

        String ruleNo = rule.getRuleNo();
        ruleMapper.deleteIndexByRuleNo(ruleNo);

        List<RuleIndex> ruleIndexList = rule.getRuleIndexList();
        ruleIndexList.forEach(ruleIndex -> ruleIndex.setRuleNo(ruleNo));
//        List<RuleIndex> list = rule.getIndexNoList().stream().map(indexNo -> new RuleIndex(rule.getRuleNo(), indexNo)).collect(Collectors.toList());
        insertIndexList(ruleIndexList);
        return ruleMapper.updateIgnoreNull(rule);

    }
	
    /**
     * 删除记录
     *
     * @param rule 待删除的记录
     * @return 返回影响行数
     */
    public int delete(Rule rule) {


        // 查询是否再其他规则集中引用
        List<String> rulesNo = ruleSetMapper.selectRulesByRuleNo(rule.getRuleNo());

        if (rulesNo == null || rulesNo.isEmpty()) {
            ruleMapper.deleteIndexByRuleNo(rule.getRuleNo());
            return ruleMapper.delete(rule);
        } else {
            throw new BusinessException("该规则已被引用，不能删除");
        }
    }


    public List<Rule> selectRuleListByRulesNo(String rulesNo) {
        return ruleMapper.selectRuleListByRulesNo(rulesNo);
    }


    /**
     * 根据规则的编号  查询关联的指标
     * @param ruleNo 规则编号
     * @return 返回集合
     */
    public List<RuleIndex> selectRuleIndexListByRuleNo(String ruleNo){
        return ruleMapper.selectRuleIndexListByRuleNo(ruleNo);
    }
	
}