package com.sundata.internalevaluation.configuration.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.sundata.common.exception.BusinessException;
import com.sundata.internalevaluation.configuration.mapper.RuleSetMapper;
import com.sundata.internalevaluation.configuration.model.RuleSet;
import com.sundata.internalevaluation.configuration.model.RuleSetRule;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author JoeLazy
 */
@Service
public class RuleSetService {

    @Autowired
    private RuleSetMapper ruleSetMapper;

    /**
     * 查询所有记录
     *
     * @return 返回集合，没有返回空List
     */
    public List<RuleSet> listAll(RuleSet ruleSet) {
        return ruleSetMapper.listAll(ruleSet);
    }


    /**
     * 根据主键查询
     *
     * @param rulesNo 主键
     * @return 返回记录，没有返回null
     */

    static final Logger log = LoggerFactory.getLogger(RuleSetService.class);

    public RuleSet getById(String rulesNo) {

        RuleSet rules = ruleSetMapper.getById(rulesNo);
        List<String> rulesIds = ruleSetMapper.selectRuleByRulesNo(rulesNo);
        rules.setRuleIds(rulesIds);
        if (rulesIds == null || rulesIds.isEmpty()) {
            log.error("暂时没有规则信息");
//            throw new BusinessException("失败了，没有规则信息");
        }
        return rules;
    }


    /**
     * 新增，忽略null字段
     *
     * @param ruleSet 新增的记录
     * @return 返回影响行数
     */
    @Transactional
    public int insertIgnoreNull(RuleSet ruleSet) throws JsonProcessingException {

        try {
            // 规则集编号
            String rulesNo = ruleSet.getRulesNo();
            // 删除绑定关系
            ruleSetMapper.deleteAssociateRuleByRulesNo(rulesNo);
            // 规则列表
            List<String> ruleIds = ruleSet.getRuleIds();
            if (ruleIds != null && !ruleIds.isEmpty()) {
                List<RuleSetRule> associateRules = ruleIds.stream().map(ruleId -> new RuleSetRule(ruleId, rulesNo)).collect(Collectors.toList());
                this.insertAssociateRule(associateRules);
            }

            return ruleSetMapper.insertIgnoreNull(ruleSet);
        } catch (DuplicateKeyException e) {
            throw new BusinessException("规则编码已存在，请重新输入.");
        }
    }

    public int insertAssociateRule(List<RuleSetRule> associateRules){
        int rows = 0;
        for (RuleSetRule associateRule : associateRules) {
            rows += ruleSetMapper.insertAssociateRule(associateRule);
        }
        return rows;
    }


    /**
     * 修改，忽略null字段
     *
     * @param ruleSet 修改的记录
     * @return 返回影响行数
     */
    @Transactional
    public int updateIgnoreNull(RuleSet ruleSet) {

        // 规则集编号
        String rulesNo = ruleSet.getRulesNo();
        // 删除绑定关系
        ruleSetMapper.deleteAssociateRuleByRulesNo(rulesNo);


        // 规则列表
        List<String> ruleIds = ruleSet.getRuleIds();
        if (ruleIds != null && !ruleIds.isEmpty()) {
            List<RuleSetRule> associateRules = ruleIds.stream().map(ruleId -> new RuleSetRule(ruleId, rulesNo)).collect(Collectors.toList());
            this.insertAssociateRule(associateRules);
        }

        return ruleSetMapper.updateIgnoreNull(ruleSet);
    }

    /**
     * 删除记录
     *
     * @param ruleSet 待删除的记录
     * @return 返回影响行数
     */

    @Transactional
    public int delete(RuleSet ruleSet) {
        ruleSetMapper.deleteAssociateRuleByRulesNo(ruleSet.getRulesNo());
        return ruleSetMapper.delete(ruleSet);
    }

}